// Testbed1.java
import java.awt.*;
import java.awt.geom.*;
import java.awt.font.*;
import javax.swing.JPanel;
import javax.swing.JFrame;

class Text
{
    String str;
    double x, y;
    double angle = 0.0;
    Text(String str, double x, double y)
    {
	this.str = str;
	this.x = x;
	this.y = y;
    }
    void setAngle(double ang)
    {
	angle = ang*Math.PI/180.0;
    }
    void draw(Graphics g)
    {
	Graphics2D g2 = (Graphics2D) g;
	Paint savePaint = g2.getPaint();
	g2.setPaint(Color.BLACK);
	AffineTransform saveXform = g2.getTransform();
        //Font font = new Font("Serif", Font.PLAIN, 16);
	Font font = g2.getFont();
	g2.translate(x,y);
	if (angle!=0.0) g2.rotate(angle);
	double scl = 1/48.;
	g2.scale(scl,-scl);
    	FontRenderContext frc = g2.getFontRenderContext();
        GlyphVector gv = font.createGlyphVector(frc, str);
        //g2.drawGlyphVector(gv, 40, 60);
        Shape shp = gv.getOutline();
    	g2.fill(shp);
	g2.setPaint(savePaint);
	g2.setTransform(saveXform);
    }
} 

public class Textbed extends JPanel
{
    final static int  resolution = 96;
    Text t1 = new Text("R1",-1.0,0.5);
    Text t2 = new Text("R2",0.5,1.0);
    Text t3 = new Text("R3",1.0,-1.0);

    Textbed()
    {
	t2.setAngle(30.0);
	t3.setAngle(90.0);
    }


    protected void show_size(Graphics g)
    {
	int width = getWidth();
	int height = getHeight();
	String str = String.format("width %d height %d",width,height);
	FontMetrics fm = g.getFontMetrics();
	Rectangle2D rect = fm.getStringBounds(str,g);
	width -= rect.getWidth();
	height -= rect.getHeight();
	g.drawString(str,width/2,height/2);
    }
    protected void draw_grid(Graphics g)
    {
	Graphics2D g2 = (Graphics2D) g;
	AffineTransform saveXform = g2.getTransform();
	Paint savePaint = g2.getPaint();
	g2.setPaint(new Color(220,220,255));
	Stroke saveStroke = g2.getStroke();
	g2.setStroke(new BasicStroke(1.0f/resolution));
	double u = 4.0;
	for (int i=-4; i<=4; i++) {
		double v = i;			
		g2.draw(new Line2D.Double(-u,v,4,v));
		g2.draw(new Line2D.Double(v,-u,v,u));
	}
	// locate origin
	boolean do_origin = true;
	if (do_origin) {
		g2.setPaint(Color.BLACK);
		u = 0.05;
		g2.draw(new Line2D.Double(-u,u,u,-u));
		g2.draw(new Line2D.Double(-u,-u,u,u));
	}
	g2.setStroke(saveStroke);
	g2.setTransform(saveXform);
    }

    public void paintComponent(Graphics g)
    {
	super.paintComponent( g );
	Graphics2D g2 = (Graphics2D) g;
	AffineTransform saveXform = g2.getTransform();
	Paint savePaint = g2.getPaint();
	int width = getWidth();
	int height = getHeight();
	g2.translate(width/2,height/2);
	g2.scale(resolution,-resolution);
	draw_grid(g);
	Stroke saveStroke = g2.getStroke();
	g2.setStroke(new BasicStroke(2.0f/resolution,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND));

	t1.draw(g);
	t2.draw(g);
	t3.draw(g);
 
	g2.setTransform(saveXform);
	g2.setPaint(savePaint);
	g2.setStroke(saveStroke);
	show_size(g);
    }


    public static void main( String args[] )
    {
	EventQueue.invokeLater(new Runnable()
	{
	    public void run()
	    {
		Textbed panel = new Textbed();
		JFrame application = new JFrame("Text Testbed");
		application.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		application.add(panel);
		application.setSize(496,358);
		application.setVisible(true);
	    }
	});
    }
}